@@ -108,7 +108,7 @@ struct acap_cb {
 struct acap_cmd_s {
     char *tag;
 
-    struct acap_requested *ask;
+    const struct acap_requested *ask;
     acap_entry_t *got;
     struct acap_cb *callbacks;
 
@@ -117,7 +117,7 @@ struct acap_cmd_s {
 
 struct acap_context_s {
     char *name;
-    struct acap_requested *ask;
+    const struct acap_requested *ask;
     struct acap_cb *callbacks;
 
     acap_context_t *next;
@@ -1246,7 +1246,7 @@ static int get_integer(acap_conn_t *conn, unsigned int *ret)
 }
 
 /* gets the entry describe in the cmd structure out */
-static int get_entry(acap_conn_t *conn, struct acap_requested *ask, 
+static int get_entry(acap_conn_t *conn, const struct acap_requested *ask, 
 		     char *name, acap_entry_t **ret)
 {
     int a = 0; /* the attribute in "ask" are we scanning */
@@ -1923,7 +1923,7 @@ int acap_process_on_command(acap_conn_t *conn, acap_cmd_t *cmd,
 
 /* ------------------- search stuff ------------------- */
 acap_context_t *acap_context_new(acap_conn_t *conn, char *name,
-				 struct acap_requested *ask)
+				 const struct acap_requested *ask)
 {
     acap_context_t *ret = (acap_context_t *) malloc(sizeof(acap_context_t));
 
@@ -1972,12 +1972,12 @@ int acap_search_dataset(acap_conn_t *conn,
 			const char *dataset, 
 			char *spec,
 			int depth,
-			struct acap_requested *ret_attrs,
-			struct acap_sort *sort_order,
-			acap_cb_completion_t *cmd_cb,
-			struct acap_search_callback *search_cb,
+			const struct acap_requested *ret_attrs,
+			const struct acap_sort *sort_order,
+			const acap_cb_completion_t *cmd_cb,
+			const struct acap_search_callback *search_cb,
 			acap_context_t **context_ptr,
-			struct acap_context_callback *context_cb,
+			const struct acap_context_callback *context_cb,
 			void *rock,
 			acap_cmd_t **ret)
 {
