@@ -192,12 +192,12 @@ int acap_search_dataset(acap_conn_t *conn,
 			const char *dataset, 
 			char *spec,
 			int depth,
-			struct acap_requested *ret_attrs,
-			struct acap_sort *sort_order,
-			acap_cb_completion_t *cmd_cb,
-			struct acap_search_callback *search_cb,
+			const struct acap_requested *ret_attrs,
+			const struct acap_sort *sort_order,
+			const acap_cb_completion_t *cmd_cb,
+			const struct acap_search_callback *search_cb,
 			acap_context_t **context_ptr,
-			struct acap_context_callback *context_cb,
+			const struct acap_context_callback *context_cb,
 			void *rock,
 			acap_cmd_t **ret);
 
