@@ -6,6 +6,15 @@
 #include "xmalloc.h"
 #include "imapconf.h"
 
+void freebuf(struct buf *buf)
+{
+    if (buf->s) {
+	free(buf->s);
+	buf->s = NULL;
+    }
+    buf->alloc = 0;
+}
+
 /*
  * Parse a word
  * (token not containing whitespace, parens, or double quotes)
