@@ -40,7 +40,7 @@
  */
 
 /*
- * $Id: pop3d.c,v 1.92 2001/05/29 20:34:14 leg Exp $
+ * $Id: pop3d.c,v 1.98 2001/07/06 23:26:10 ken3 Exp $
  */
 #include <config.h>
 
@@ -53,6 +53,7 @@
 #include <fcntl.h>
 #include <time.h>
 #include <signal.h>
+#include <assert.h>
 #include <sys/types.h>
 #include <sys/param.h>
 #include <syslog.h>
@@ -122,6 +123,7 @@ struct msg {
     int deleted;
 } *popd_msg;
 
+static int pop3s = 0;
 int popd_starttls_done = 0;
 
 static struct mailbox mboxstruct;
@@ -157,6 +159,42 @@ static struct sasl_callback mysasl_cb[] = {
     { SASL_CB_LIST_END, NULL, NULL }
 };
 
+static void popd_reset(void)
+{
+    proc_cleanup();
+
+    if (popd_mailbox) {
+	mailbox_close(popd_mailbox);
+	popd_mailbox = 0;
+    }
+
+    prot_flush(popd_out);
+    if (popd_in) prot_free(popd_in);
+    if (popd_out) prot_free(popd_out);
+    close(0);
+    close(1);
+    close(2);
+
+    strcpy(popd_clienthost, "[local]");
+    if (popd_userid != NULL) {
+	free(popd_userid);
+	popd_userid = NULL;
+    }
+    if (popd_saslconn) {
+	sasl_dispose(&popd_saslconn);
+	popd_saslconn = NULL;
+    }
+    popd_starttls_done = 0;
+#ifdef HAVE_SSL
+    if (tls_conn) {
+	tls_free(&tls_conn);
+	tls_conn = NULL;
+    }
+#endif
+
+    popd_exists = 0;
+}
+
 /*
  * run once when process is forked;
  * MUST NOT exit directly; must return with non-zero error code
@@ -164,6 +202,7 @@ static struct sasl_callback mysasl_cb[] = {
 int service_init(int argc, char **argv, char **envp)
 {
     int r;
+    int opt;
 
     config_changeident("pop3d");
     if (geteuid() == 0) fatal("must run as the Cyrus user", EC_USAGE);
@@ -194,26 +233,6 @@ int service_init(int argc, char **argv, char **envp)
     /* setup for sending IMAP IDLE notifications */
     idle_enabled();
 
-    return 0;
-}
-
-/*
- * run for each accepted connection
- */
-int service_main(int argc, char **argv, char **envp)
-{
-    int pop3s = 0;
-    int opt;
-    socklen_t salen;
-    struct hostent *hp;
-    int timeout;
-    sasl_security_properties_t *secprops=NULL;
-
-    signals_poll();
-
-    popd_in = prot_new(0, 0);
-    popd_out = prot_new(1, 1);
-
     while ((opt = getopt(argc, argv, "C:sk")) != EOF) {
 	switch(opt) {
 	case 'C': /* alt config file - handled by service::main() */
@@ -236,6 +255,24 @@ int service_main(int argc, char **argv, char **envp)
 	}
     }
 
+    return 0;
+}
+
+/*
+ * run for each accepted connection
+ */
+int service_main(int argc, char **argv, char **envp)
+{
+    socklen_t salen;
+    struct hostent *hp;
+    int timeout;
+    sasl_security_properties_t *secprops=NULL;
+
+    signals_poll();
+
+    popd_in = prot_new(0, 0);
+    popd_out = prot_new(1, 1);
+
     /* Find out name of client host */
     salen = sizeof(popd_remoteaddr);
     if (getpeername(0, (struct sockaddr *)&popd_remoteaddr, &salen) == 0 &&
@@ -288,6 +325,14 @@ int service_main(int argc, char **argv, char **envp)
 		CYRUS_VERSION);
     cmdloop();
 
+    /* QUIT executed */
+
+    /* don't bother reusing KPOP connections */
+    if (kflag) shut_down(0);
+
+    /* cleanup */
+    popd_reset();
+
     return 0;
 }
 
@@ -446,7 +491,7 @@ static void cmdloop(void)
 		    }
 		}
 		prot_printf(popd_out, "+OK\r\n");
-		shut_down(0);
+		return;
 	    }
 	    else prot_printf(popd_out, "-ERR Unexpected extra argument\r\n");
 	}
@@ -467,7 +512,8 @@ static void cmdloop(void)
 	}
 	else if (!popd_mailbox) {
 	    if (!strcmp(inputbuf, "user") &&
-		(popd_starttls_done || config_getswitch("allowplaintext", 1))) {
+		(kflag || popd_starttls_done ||
+		 config_getswitch("allowplaintext", 1))) {
 		if (!arg) {
 		    prot_printf(popd_out, "-ERR Missing argument\r\n");
 		}
@@ -476,27 +522,31 @@ static void cmdloop(void)
 		}
 	    }
 	    else if (!strcmp(inputbuf, "pass") &&
-		     (popd_starttls_done || config_getswitch("allowplaintext", 1))) {
+		     (kflag || popd_starttls_done ||
+		      config_getswitch("allowplaintext", 1))) {
 		if (!arg) prot_printf(popd_out, "-ERR Missing argument\r\n");
 		else cmd_pass(arg);
 	    }
 	    else if (!strcmp(inputbuf, "apop") && apop_enabled()) {
 		char *user = NULL, *digest = NULL;
 
-		/* Parse into user and digest */
+		/* Parse out username and digest.
+		 *
+		 * Per RFC 1939, response must be "<user> <digest>", where
+		 * <digest> is a 16-octet value which is sent in hexadecimal
+		 * format, using lower-case ASCII characters.
+		 */
 		if (arg) {
 		    user = arg;
-		    arg = strchr(arg, ' ');
+		    arg = strrchr(arg, ' ');
 		}
 		if (!arg) prot_printf(popd_out, "-ERR Missing argument\r\n");
+		else if (strspn(arg + 1, "0123456789abcdef") != 32)
+		    prot_printf(popd_out, "-ERR Invalid digest string\r\n");
 		else {
 		    *arg++ = '\0';
 		    digest = arg;
-		    /* per RFC 1939, digest must be 32 hex digits */
-		    while (*arg && isxdigit((int) *arg)) arg++;
-		    if ((arg - digest) != 32)
-			prot_printf(popd_out, "-ERR Invalid digest string\r\n");
-		    else cmd_apop(user, digest);
+		    cmd_apop(user, digest);
 		}
 	    }
 	    else if (!strcmp(inputbuf, "auth")) {
@@ -1006,7 +1056,7 @@ cmd_capa()
     prot_printf(popd_out, "PIPELINING\r\n");
     prot_printf(popd_out, "RESP-CODES\r\n");
 
-    if (popd_starttls_done || config_getswitch("allowplaintext", 1)) {
+    if (kflag || popd_starttls_done || config_getswitch("allowplaintext", 1)) {
 	prot_printf(popd_out, "USER\r\n");
     }
     
@@ -1032,6 +1082,7 @@ void cmd_auth(char *arg)
     char *serverout;
     unsigned int serveroutlen;
     const char *errstr;
+    char *canon_user;
 
     /* if client didn't specify an argument we give them the list */
     if (!arg) {
@@ -1133,7 +1184,8 @@ void cmd_auth(char *arg)
      * mysasl_authproc()
      */
     sasl_result = sasl_getprop(popd_saslconn, SASL_USERNAME,
-			       (void **) &popd_userid);
+			       (void **) &canon_user);
+    popd_userid = xstrdup(canon_user);
     if (sasl_result != SASL_OK) {
 	prot_printf(popd_out, 
 		    "-ERR weird SASL error %d getting SASL_USERNAME\r\n", 
