@@ -39,7 +39,7 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: reconstruct.c,v 1.57 2001/01/09 00:19:04 leg Exp $ */
+/* $Id: reconstruct.c,v 1.60 2001/03/14 05:56:49 leg Exp $ */
 
 #include <config.h>
 
@@ -110,7 +110,6 @@ int reconstruct(char *name, struct discovered *l);
 void usage(void);
 void shut_down(int code);
 
-extern char *mailbox_findquota(const char *name);
 extern cyrus_acl_canonproc_t mboxlist_ensureOwnerRights;
 
 static acapmbox_handle_t *acaphandle = NULL;
@@ -125,9 +124,9 @@ int main(int argc, char **argv)
     int fflag = 0;
     char buf[MAX_MAILBOX_PATH];
     struct discovered head;
+    char *alt_config = NULL;
 
     memset(&head, 0, sizeof(head));
-    config_init("reconstruct");
 
     if (geteuid() == 0) fatal("must run as the Cyrus user", EC_USAGE);
 
@@ -135,8 +134,12 @@ int main(int argc, char **argv)
     assert(INDEX_HEADER_SIZE == (OFFSET_FLAGGED+4));
     assert(INDEX_RECORD_SIZE == (OFFSET_USER_FLAGS+MAX_USER_FLAGS/8));
 
-    while ((opt = getopt(argc, argv, "rmf")) != EOF) {
+    while ((opt = getopt(argc, argv, "C:rmf")) != EOF) {
 	switch (opt) {
+	case 'C': /* alt config file */
+	    alt_config = optarg;
+	    break;
+
 	case 'r':
 	    rflag = 1;
 	    break;
@@ -154,6 +157,8 @@ int main(int argc, char **argv)
 	}
     }
 
+    config_init(alt_config, "reconstruct");
+
     if (mflag) {
 	if (rflag || fflag || optind != argc) usage();
 	do_mboxlist();
@@ -197,7 +202,11 @@ int main(int argc, char **argv)
 
 	/* create p and reconstruct it */
 	r = mboxlist_createmailbox(p->name, 0, NULL, 1, "cyrus", NULL);
-	if (!r) do_reconstruct(p->name, 0, 0, &head);
+	if (!r) {
+	    do_reconstruct(p->name, 0, 0, &head);
+	} else {
+	    printf("createmailbox %s: %s\n", p->name, error_message(r));
+	}
 	/* may have added more things into our list */
 
 	free(p->name);
@@ -212,8 +221,9 @@ int main(int argc, char **argv)
 
 void usage(void)
 {
-    fprintf(stderr, "usage: reconstruct [-r] [-f] mailbox...\n");
-    fprintf(stderr, "       reconstruct -m\n");
+    fprintf(stderr,
+	    "usage: reconstruct [-C <alt_config>] [-r] [-f] mailbox...\n");
+    fprintf(stderr, "       reconstruct [-C <alt_config>] -m\n");
     exit(EC_USAGE);
 }    
 
@@ -253,7 +263,8 @@ int reconstruct(char *name, struct discovered *found)
 {
     int r;
     struct mailbox mailbox;
-    char *quota_root;
+    char quota_root[MAX_MAILBOX_PATH];
+    int hasquota;
     int i, flag;
     char *p;
     int format = MAILBOX_FORMAT_NORMAL;
@@ -295,9 +306,9 @@ int reconstruct(char *name, struct discovered *found)
     }
 
     /* Fix quota root */
-    quota_root = mailbox_findquota(mailbox.name);
+    hasquota = mailbox_findquota(quota_root, mailbox.name);
     if (mailbox.quota.root) free(mailbox.quota.root);
-    if (quota_root) {
+    if (hasquota) {
 	mailbox.quota.root = xstrdup(quota_root);
     }
     else {
