@@ -1,5 +1,5 @@
 /* seen_db.c -- implementation of seen database using per-user berkeley db
-   $Id: seen_db.c,v 1.17 2000/10/12 19:18:30 leg Exp $
+   $Id: seen_db.c,v 1.21 2001/01/05 06:00:03 leg Exp $
  
  * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -75,11 +75,12 @@ enum {
 };
 
 struct seen {
-    const char *user;		/* what user is this for? */
+    char *user;			/* what user is this for? */
     const char *uniqueid;	/* what mailbox? */
     const char *path;		/* where is this mailbox? */
     struct db *db;
     struct txn *tid;		/* outstanding txn, if any */
+    int converting;
 };
 
 static struct seen *lastseen = NULL;
@@ -133,6 +134,7 @@ int seen_open(struct mailbox *mailbox,
     if (seendb && !strcmp(seendb->user, user)) {
 	abortcurrent(seendb);
 	seendb->uniqueid = mailbox->uniqueid;
+	seendb->path = mailbox->path;
 	*seendbptr = seendb;
 	return 0;
     }
@@ -146,6 +148,7 @@ int seen_open(struct mailbox *mailbox,
 	    syslog(LOG_ERR, "DBERROR: error closing seendb: %s", 
 		   cyrusdb_strerror(r));
 	}
+	free(seendb->user);
     } else {
 	/* create seendb */
 	seendb = (struct seen *) xmalloc(sizeof(struct seen));
@@ -168,7 +171,7 @@ int seen_open(struct mailbox *mailbox,
     seendb->tid = NULL;
     seendb->uniqueid = mailbox->uniqueid;
     seendb->path = mailbox->path;
-    seendb->user = user;
+    seendb->user = xstrdup(user);
 
     *seendbptr = seendb;
     return r;
@@ -195,28 +198,32 @@ static int seen_readold(struct seen *seendb,
     strcat(fnamebuf, FNAME_SEEN);
 
     fd = open(fnamebuf, O_RDWR, 0);
-    if (fd == -1 && errno == ENOENT) {
-	/* no old-style seen file for this database */
-	linelen = 0;
-    } else if (fd == -1) {
-	syslog(LOG_ERR, "error opening '%s': %m", fnamebuf);
-	return IMAP_IOERROR;
-    } else {
-	if (fstat(fd, &sbuf) == -1) {
-	    close(fd);
-	    return IMAP_IOERROR;
-	}
-	map_refresh(fd, 1, &base, &len, sbuf.st_size, fnamebuf, 0);
-	
-	/* Find record for user */
-	offset = bsearch_mem(seendb->user, 1, base, len, 0, &linelen);
-    }
 
     *lastreadptr = 0;
     *lastuidptr = 0;
     *lastchangeptr = 0;
+
+    if (fd == -1 && errno == ENOENT) {
+	/* no old-style seen file for this database */
+	*seenuidsptr = xstrdup("");
+	return 0;
+    } else if (fd == -1) {
+	syslog(LOG_ERR, "error opening '%s': %m", fnamebuf);
+	return IMAP_IOERROR;
+    }
+
+    if (fstat(fd, &sbuf) == -1) {
+	close(fd);
+	return IMAP_IOERROR;
+    }
+    map_refresh(fd, 1, &base, &len, sbuf.st_size, fnamebuf, 0);
+    
+    /* Find record for user */
+    offset = bsearch_mem(seendb->user, 1, base, len, 0, &linelen);
+
     if (!linelen) {
 	*seenuidsptr = xstrdup("");
+	close(fd);
 	return 0;
     }
 
@@ -253,7 +260,6 @@ static int seen_readit(struct seen *seendb,
     int uidlen;
 
     assert(seendb && seendb->uniqueid);
-
     if (rw) {
 	r = DB->fetchlock(seendb->db, 
 			  seendb->uniqueid, strlen(seendb->uniqueid),
@@ -277,8 +283,9 @@ static int seen_readit(struct seen *seendb,
 	break;
     }
     if (data == NULL) {
-	return seen_readold(seendb, lastreadptr, lastuidptr,
-			    lastchangeptr, seenuidsptr);
+	r = seen_readold(seendb, lastreadptr, lastuidptr,
+			 lastchangeptr, seenuidsptr);
+	return r;
     }
 
     /* remember that 'data' may not be null terminated ! */
@@ -316,8 +323,6 @@ int seen_lockread(struct seen *seendb,
 		  time_t *lastreadptr, unsigned int *lastuidptr, 
 		  time_t *lastchangeptr, char **seenuidsptr)
 {
-    assert(seendb && seendb->uniqueid);
-
     if (SEEN_DEBUG) {
 	syslog(LOG_DEBUG, "seen_db: seen_lockread(%s, %s)", 
 	       seendb->uniqueid, seendb->user);
@@ -348,7 +353,7 @@ int seen_write(struct seen *seendb, time_t lastread, unsigned int lastuid,
     datalen = strlen(data);
 
     r = DB->store(seendb->db, seendb->uniqueid, strlen(seendb->uniqueid),
-		  data, datalen, NULL);
+		  data, datalen, &seendb->tid);
     switch (r) {
     case CYRUSDB_OK:
 	break;
@@ -368,13 +373,25 @@ int seen_write(struct seen *seendb, time_t lastread, unsigned int lastuid,
 
 int seen_close(struct seen *seendb)
 {
+    int r;
+
     if (SEEN_DEBUG) {
 	syslog(LOG_DEBUG, "seen_db: seen_close(%s, %s)", 
 	       seendb->uniqueid, seendb->user);
     }
 
-    abortcurrent(seendb);
+    if (seendb->tid) {
+	r = DB->commit(seendb->db, seendb->tid);
+	if (r != CYRUSDB_OK) {
+	    syslog(LOG_ERR, "DBERROR: error committing seen txn; "
+		   "seen state lost: %s", cyrusdb_strerror(r));
+	    DB->abort(seendb->db, seendb->tid);
+	}
+	seendb->tid = NULL;
+    }
+
     seendb->uniqueid = NULL;
+    seendb->path = NULL;
 
     if (lastseen) {
 	int r;
@@ -387,7 +404,7 @@ int seen_close(struct seen *seendb)
 		   cyrusdb_strerror(r));
 	    r = IMAP_IOERROR;
 	}
-
+	free(lastseen->user);
 	free(lastseen);
     }
 
@@ -461,17 +478,27 @@ int seen_copy(struct mailbox *oldmailbox, struct mailbox *newmailbox)
     return 0;
 }
 
+/* database better have been locked before this ! */
 int seen_unlock(struct seen *seendb)
 {
+    int r;
+
     assert(seendb);
+    if (!seendb->tid) return 0;
 
     if (SEEN_DEBUG) {
 	syslog(LOG_DEBUG, "seen_db: seen_unlock(%s, %s)",
 	       seendb->uniqueid, seendb->user);
     }
 
-    abortcurrent(seendb);
-    /* we lazily close the database */
+    r = DB->commit(seendb->db, seendb->tid);
+    if (r != CYRUSDB_OK) {
+	syslog(LOG_ERR, "DBERROR: error committing seen txn; "
+	       "seen state lost: %s", cyrusdb_strerror(r));
+	DB->abort(seendb->db, seendb->tid);
+    }
+    seendb->tid = NULL;
+
     return 0;
 }
 
@@ -491,6 +518,7 @@ int seen_done(void)
 		   cyrusdb_strerror(r));
 	    r = IMAP_IOERROR;
 	}
+	free(lastseen->user);
 	free(lastseen);
     }
 
