@@ -39,7 +39,7 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: master.c,v 1.40 2001/06/23 00:59:48 ken3 Exp $ */
+/* $Id: master.c,v 1.42 2001/07/16 18:23:11 leg Exp $ */
 
 #include <config.h>
 
@@ -326,16 +326,19 @@ void service_create(struct service *s)
         if ((port = parse_listen(listen)) == NULL) {
             /* listen IS the port */
             if (!resolve_port(listen, s, &sin)) {
+		free(listen);
                 return;
             }
             sin.sin_addr.s_addr = INADDR_ANY;
         } else {
             /* listen is now just the address */
             if (!resolve_port(port, s, &sin)) {
+		free(listen);
                 return;
             }
             if (!resolve_host(listen, &sin)) {
 		s->exec = NULL;
+		free(listen);
                 return;
             }
         }
@@ -354,7 +357,11 @@ void service_create(struct service *s)
     }
 
     /* allow reuse of address */
-    setsockopt(s->socket, SOL_SOCKET, SO_REUSEADDR, (void *) &on, sizeof(on));
+    r = setsockopt(s->socket, SOL_SOCKET, SO_REUSEADDR, 
+		   (void *) &on, sizeof(on));
+    if (r < 0) {
+	syslog(LOG_ERR, "unable to setsocketopt(SO_REUSEADDR): %m");
+    }
 
     oldumask = umask((mode_t) 0); /* for linux */
     r = bind(s->socket, sa, salen);
