@@ -1,5 +1,5 @@
 
-/*  A Bison parser, made from ./addr.y
+/*  A Bison parser, made from addr.y
     by GNU Bison version 1.28  */
 
 #define YYBISON 1  /* Identify Bison output.  */
@@ -15,7 +15,7 @@
 #define	QTEXT	258
 #define	DTEXT	259
 
-#line 1 "./addr.y"
+#line 1 "addr.y"
 
 /*
  * addr.y -- RFC 822 address parser
@@ -934,7 +934,7 @@ yyerrhandle:
     }
   return 1;
 }
-#line 85 "./addr.y"
+#line 85 "addr.y"
 
 
 /* copy address error message into buffer provided by sieve parser */
