@@ -71,6 +71,7 @@
 #include "prot.h"
 #include "imapconf.h"
 #include "xmalloc.h"
+#include "exitcodes.h"
 
 #include "codes.h"
 #include "actions.h"
@@ -124,8 +125,7 @@ void fatal(const char *s, int code)
     prot_printf(sieved_out, "NO Fatal error: %s\r\n", s);
     prot_flush(sieved_out);
 
-    exit(1);
-
+    exit(EC_TEMPFAIL);
 }
 
 /* should we allow users to proxy?  return SASL_OK if yes,
@@ -229,6 +229,7 @@ int service_main(int argc, char **argv, char **envp)
     socklen_t salen;
     struct hostent *hp;
     int timeout;
+    int secflags = 0;
     sasl_security_properties_t *secprops = NULL;
 
     /* set up the prot streams */
@@ -284,7 +285,11 @@ int service_main(int argc, char **argv, char **envp)
 
     /* will always return something valid */
     /* should be configurable! */
-    secprops = mysasl_secprops(SASL_SEC_NOANONYMOUS);
+    secflags = SASL_SEC_NOANONYMOUS;
+    if (!config_getswitch("allowplaintext", 1)) {
+	secflags |= SASL_SEC_NOPLAINTEXT;
+    }
+    secprops = mysasl_secprops(secflags);
     sasl_setprop(sieved_saslconn, SASL_SEC_PROPS, secprops);
     
     sasl_setprop(sieved_saslconn, SASL_IP_REMOTE, &sieved_remoteaddr);  
@@ -296,7 +301,7 @@ int service_main(int argc, char **argv, char **envp)
     cmdloop();
 
     /* never reaches */
-    exit(-99);
+    exit(EC_SOFTWARE);
 }
 
 
